package com.bocloud.cmp.config;

import com.bocloud.cmp.boot.model.BoCloudExchange;
import com.bocloud.cmp.listener.BocloudMessageListener;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.core.CurrentService;
import org.springframework.amqp.core.*;
import org.springframework.amqp.rabbit.connection.ConnectionFactory;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.amqp.rabbit.listener.SimpleMessageListenerContainer;
import org.springframework.amqp.support.converter.Jackson2JsonMessageConverter;
import org.springframework.amqp.support.converter.MessageConverter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Scope;

import java.util.Map;

/**
 * rabbitmq配置类
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年3月30日
 */
@Configuration
@ConfigurationProperties(prefix = "spring.rabbitmq")
public class RabbitMQConfiguration {

    private final ConnectionFactory connectionFactory;
    private final Map<String, Object> arguments = MapTools.simpleMap("x-ha-policy", "all");

    @Autowired
    public RabbitMQConfiguration(ConnectionFactory connectionFactory) {
        this.connectionFactory = connectionFactory;
    }

    /**
     * ==============================================================================================================
     * 定义direct exchange
     *
     * @return
     */
    @Bean
    public DirectExchange declareDirectExchange() {
        return new DirectExchange("bocloud.direct.exchange", true, true, arguments);
    }

    /**
     * 声明主题交换
     *
     * @return
     */
    @Bean
    public TopicExchange topicExchange() {
        return new TopicExchange(BoCloudExchange.Default.TOPIC.getExchange(), true, true);
    }

    /**
     * 绑定资源申请流程队列
     *
     * @return
     */
    @Bean
    public Binding bindingMessageQueue(CurrentService service) {
        return BindingBuilder.bind(declareMessageQueue(service)).to(topicExchange()).with("bocloud.message.queue.#");
    }

    @Bean
    public MessageConverter messageConverter() {
        return new Jackson2JsonMessageConverter();
    }

    @Bean
    public Queue declareMessageQueue(CurrentService service) {
        return new Queue("bocloud.message.queue.sms." + service.getToken(), true, false, true, arguments);
    }

    @Bean
    @Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
    public RabbitTemplate amqpTemplate() {
        RabbitTemplate amqpTemplate = new RabbitTemplate(connectionFactory);
        amqpTemplate.setMessageConverter(messageConverter());
        return amqpTemplate;
    }

    /**
     * 声明审计日志队列
     *
     * @return
     */
    @Bean
    public Queue declareLogQueue() {
        return new Queue("bocloud.log.queue", true, false, false, arguments);
    }

    /**
     * 声明通知消息队列
     *
     * @return
     */
    @Bean
    public Queue declareNoticeQueue() {
        return new Queue("event.notice.queue", true, false, false, arguments);
    }

    /**
     * 声明资源订购队列
     *
     * @return
     */
    @Bean
    public Queue declareIaasCreationQueue() {
        return new Queue("iaas.resource.creation.queue", true, false, false, arguments);
    }

    /**
     * 绑定资源订购队列
     *
     * @return
     */
    @Bean
    public Binding bindingIaasCreationQueue() {
        return BindingBuilder.bind(declareIaasCreationQueue()).to(declareDirectExchange()).withQueueName();
    }

    /**
     * 声明资源变更队列
     *
     * @return
     */
    @Bean
    public Queue declareIaasModificationQueue() {
        return new Queue("iaas.resource.modification.queue", true, false, false, arguments);
    }

    /**
     * 绑定资源变更队列
     *
     * @return
     */
    @Bean
    public Binding bindingIaasModificationQueue() {
        return BindingBuilder.bind(declareIaasModificationQueue()).to(declareDirectExchange()).withQueueName();
    }

    /**
     * 声明资源退订队列
     *
     * @return
     */
    @Bean
    public Queue declareIaasDeletionQueue() {
        return new Queue("iaas.resource.deletion.queue", true, false, false, arguments);
    }

    /**
     * 声明资源延期队列
     *
     * @return
     */
    @Bean
    public Queue declareIaasExtensionQueue() {
        return new Queue("iaas.resource.extension.queue", true, false, false, arguments);
    }

    /**
     * 绑定资源退订队列
     *
     * @return
     */
    @Bean
    public Binding bindingIaasDeletionQueue() {
        return BindingBuilder.bind(declareIaasDeletionQueue()).to(declareDirectExchange()).withQueueName();
    }

    /**
     * 绑定资源延期队列
     *
     * @return
     */
    @Bean
    public Binding bindingIaasExtensionQueue() {
        return BindingBuilder.bind(declareIaasExtensionQueue()).to(declareDirectExchange()).withQueueName();
    }

    /**
     * 声明云平台统计队列
     *
     * @return
     */
    @Bean
    public Queue declareStatisticsQueue() {
        return new Queue("vendor.statistics.queue", true, false, false, arguments);
    }

    /**
     * 绑定云平台统计队列
     *
     * @return
     */
    @Bean
    public Binding bindingStatisticsQueue() {
        return BindingBuilder.bind(declareStatisticsQueue()).to(declareDirectExchange()).withQueueName();
    }

    /**
     * ROS编排队列
     * 
     * @return
     */
    @Bean
    public Queue declareForRosQueue() {
        return new Queue("bocloud.ros.queue", true, false, false, arguments);
    }

    @Bean
    public Binding bindingForRosQueue() {
        return BindingBuilder.bind(declareForRosQueue()).to(declareDirectExchange()).withQueueName();
    }

    /**
     * ROS编排回滚队列
     * 
     * @return
     */
    @Bean
    public Queue declareForRosCallbackQueue() {
        return new Queue("ros.callback.queue", true, false, false, arguments);
    }

    @Bean
    public Binding bindingForRosCallbackQueue() {
        return BindingBuilder.bind(declareForRosCallbackQueue()).to(declareDirectExchange()).withQueueName();
    }

    /**
     * ROS编排软件回滚队列
     * 
     * @return
     */
    @Bean
    public Queue declareForSoftCallbackQueue() {
        return new Queue("software.callback.queue", true, false, false, arguments);
    }

    @Bean
    public Binding bindingForSoftCallbackQueue() {
        return BindingBuilder.bind(declareForSoftCallbackQueue()).to(declareDirectExchange()).withQueueName();
    }

    /**
     * @Author: lxy
     * @Description: 堡垒机接收创建vm回复信息
     * @Date 2021/5/13 17:33
     * @Param
     * @return org.springframework.amqp.core.Queue
     * @throws:
     **/
    @Bean
    public Queue declareSyncShtermCallbackQueue() {
        return new Queue("sync.shterm.callback.queue", true, false, false, arguments);
    }

    @Bean
    public Binding bindingSyncShtermCallbackQueue() {
        return BindingBuilder.bind(declareSyncShtermCallbackQueue()).to(declareDirectExchange()).withQueueName();
    }

    /**
     * 堡垒机部门同步消息回调队列
     *
     * @return
     */
    @Bean
    public Queue declareFortressCallbackQueueDept() {
        return new Queue("fortress.callback.queue.dept", true, false, false, arguments);
    }

    @Bean
    public Binding bindingFortressCallbackQueueDept() {
        return BindingBuilder.bind(declareFortressCallbackQueueDept()).to(declareDirectExchange()).withQueueName();
    }

    /**
     * 用于cop主机统计
     * 
     * @return
     */
    @Bean
    public Queue declareForCopTaskHostQueue() {
        return new Queue("bocloud.taskhost.operate", true, false, false, arguments);
    }

    @Bean
    public Binding bindingForCopTaskHostQueue() {
        return BindingBuilder.bind(declareForCopTaskHostQueue()).to(declareDirectExchange()).withQueueName();
    }

    /**
     * 堡垒机项目同步消息回调队列
     *
     * @return
     */
    @Bean
    public Queue declareFortressCallbackQueueProject() {
        return new Queue("fortress.callback.queue.project", true, false, false, arguments);
    }

    @Bean
    public Binding bindingFortressCallbackQueueProject() {
        return BindingBuilder.bind(declareFortressCallbackQueueProject()).to(declareDirectExchange()).withQueueName();
    }

    /**
     * 堡垒机用户同步消息回调队列
     *
     * @return
     */
    @Bean
    public Queue declareFortressCallbackQueueUser() {
        return new Queue("fortress.callback.queue.user", true, false, false, arguments);
    }

    @Bean
    public Binding bindingFortressCallbackQueueUser() {
        return BindingBuilder.bind(declareFortressCallbackQueueUser()).to(declareDirectExchange()).withQueueName();
    }

    /**
     * 堡垒机部门、资源组、用户同步队列
     *
     * @return
     */
    @Bean
    public Queue declareFortressSyncQueue() {
        return new Queue("fortress.sync.queue", true, false, false, arguments);
    }

    @Bean
    public Binding bindingFortressSyncQueue() {
        return BindingBuilder.bind(declareFortressSyncQueue()).to(declareDirectExchange()).withQueueName();
    }

    /**
     * 声明Exporter安装结果队列
     *
     * @return
     */
    @Bean
    public Queue declareExporterTaskQueue() {
        return new Queue("exporter.task.result.queue", true, false, false, arguments);
    }

    /**
     * 绑定Exporter安装作业结果队列
     *
     * @return
     */
    @Bean
    public Binding bindingExporterTaskQueue() {
        return BindingBuilder.bind(declareExporterTaskQueue()).to(declareDirectExchange()).withQueueName();
    }

    /**
     * 声明任务服务结果请求队列
     *
     * @return
     */
    @Bean
    public Queue declareServiceResultQueue() {
        return new Queue("ros.service.result", true, false, false, arguments);
    }

    /**
     * 资源任务服务结果请求队列绑定exchange
     *
     * @return
     */
    @Bean
    public Binding bindingServiceResultQueue() {
        return BindingBuilder.bind(declareServiceResultQueue()).to(declareDirectExchange()).withQueueName();
    }

    /**
     * 声明资源订购队列
     *
     * @return
     */
    @Bean
    public Queue declarePaasCreationQueue() {
        return new Queue("paas.resource.creation.queue", true, false, false, arguments);
    }

    /**
     * 绑定资源订购队列
     *
     * @return
     */
    @Bean
    public Binding bindingPaasCreationQueue() {
        return BindingBuilder.bind(declarePaasCreationQueue()).to(declareDirectExchange()).withQueueName();
    }

    /**
     * 声明资源变更队列
     *
     * @return
     */
    @Bean
    public Queue declarePaasModificationQueue() {
        return new Queue("paas.resource.modification.queue", true, false, false, arguments);
    }

    /**
     * 绑定资源变更队列
     *
     * @return
     */
    @Bean
    public Binding bindingPaasModificationQueue() {
        return BindingBuilder.bind(declarePaasModificationQueue()).to(declareDirectExchange()).withQueueName();
    }

    /**
     * 声明资源退订队列
     *
     * @return
     */
    @Bean
    public Queue declarePaasDeletionQueue() {
        return new Queue("paas.resource.deletion.queue", true, false, false, arguments);
    }

    /**
     * 绑定资源退订队列
     *
     * @return
     */
    @Bean
    public Binding bindingDeletionQueue() {
        return BindingBuilder.bind(declarePaasDeletionQueue()).to(declareDirectExchange()).withQueueName();
    }

    /**
     * 声明资源延期队列
     *
     * @return
     */
    @Bean
    public Queue declarePaasExtensionQueue() {
        return new Queue("paas.resource.extension.queue", true, false, false, arguments);
    }

    /**
     * 绑定资源延期队列
     *
     * @return
     */
    @Bean
    public Binding bindingPaasExtensionQueue() {
        return BindingBuilder.bind(declarePaasExtensionQueue()).to(declareDirectExchange()).withQueueName();
    }

    /**
     * 声明软件安装消息队列
     *
     * @return
     */
    @Bean
    public Queue declareSoftInstallQueue() {
        return new Queue("software.install.queue", true, false, false, arguments);
    }

    /**
     * 资源软件安装消息队列绑定exchange
     *
     * @return
     */
    @Bean
    public Binding bindingSoftInstallQueue() {
        return BindingBuilder.bind(declareSoftInstallQueue()).to(declareDirectExchange()).withQueueName();
    }

    /**
     * 声明软件事件队列
     *
     * @return
     */
    @Bean
    public Queue declareSoftEventQueue() {
        return new Queue("software.event.queue", true, false, false, arguments);
    }

    /**
     * 软件事件列绑定exchange
     *
     * @return
     */
    @Bean
    public Binding bindingSoftEventQueue() {
        return BindingBuilder.bind(declareSoftEventQueue()).to(declareDirectExchange()).withQueueName();
    }

    /**
     * 声明集群节点变更队列
     *
     * @return
     */
    @Bean
    public Queue declareNodeAlterationQueue() {
        return new Queue("software.node.event", true, false, false, arguments);
    }

    /**
     * 绑定集群节点变更队列
     *
     * @return
     */
    @Bean
    public Binding bindingNodeAlterationQueue() {
        return BindingBuilder.bind(declareNodeAlterationQueue()).to(declareDirectExchange()).withQueueName();
    }

    @Bean
    public RabbitAdmin rabbitAdmin() {
        return new RabbitAdmin(amqpTemplate());
    }

    @Bean
    public SimpleMessageListenerContainer autoMessageListener(CurrentService service,
        BocloudMessageListener bocloudMessageListener) {
        SimpleMessageListenerContainer container = new SimpleMessageListenerContainer(connectionFactory);
        container.setQueues(declareMessageQueue(service));
        // 设置确认模式自动确认
        container.setAcknowledgeMode(AcknowledgeMode.AUTO);
        container.setMessageListener(bocloudMessageListener);
        container.setAmqpAdmin(rabbitAdmin());
        container.setAutoStartup(true);
        return container;
    }

}
