package com.bocloud.cmp.config;

import com.alibaba.fastjson.serializer.SerializerFeature;
import com.alibaba.fastjson.support.config.FastJsonConfig;
import com.alibaba.fastjson.support.spring.FastJsonHttpMessageConverter;
import com.bocloud.cmp.boot.config.PagerArgumentResolver;
import com.bocloud.cmp.boot.interceptor.AccessInterceptor;
import com.bocloud.cmp.boot.interceptor.LogInterceptor;
import com.megatron.framework.core.InterceptorBridge;
import com.megatron.framework.core.config.InterceptorConfiguration;
import com.megatron.framework.core.domain.InterceptorEntity;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.http.converter.ByteArrayHttpMessageConverter;
import org.springframework.http.converter.HttpMessageConverter;
import org.springframework.stereotype.Component;
import org.springframework.web.method.support.HandlerMethodArgumentResolver;

import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;

/**
 * 拦截器配置类
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年3月30日
 */
@Slf4j
@Component
public class WebInterceptorConfiguration implements InterceptorConfiguration {

    private static final String[] EXCLUDES = new String[]{"/status**", "/doc.html", "/webjars/**", "/swagger-resources/**",
            "**/api-docs", "**/status/**", "/v2/**", "/webjars/**", "/*/*.css", "/*/*.png", "/*.html", "/*/*.html",
            "/*/*.js", "/*/*.jpg", "/*/*.jpeg", "/login", "/sso", "/v1/logview", "/v1/logzip", "/v1/test/**",
            "/v3/api-docs/**", "/v1/bsm/**"};
    private static final String[] INCLUDES = new String[]{"/**"};
    private final LogInterceptor logInterceptor;
    private final AccessInterceptor accessInterceptor;

    @Autowired
    public WebInterceptorConfiguration(AccessInterceptor accessInterceptor, LogInterceptor logInterceptor) {
        this.logInterceptor = logInterceptor;
        this.accessInterceptor = accessInterceptor;
    }


    @Override
    public InterceptorBridge buildBridge() {
        log.info("start to build interceptor bridge ...");

        InterceptorEntity accessEntity = InterceptorEntity.builder().interceptor(accessInterceptor).includes(INCLUDES).excludes(EXCLUDES).order(0).build();
        InterceptorEntity auditLogEntity = InterceptorEntity.builder().interceptor(logInterceptor).includes(INCLUDES).excludes(EXCLUDES).order(1).build();

        List<InterceptorEntity> interceptors = new ArrayList<>();
        interceptors.add(accessEntity);
        interceptors.add(auditLogEntity);

        List<HttpMessageConverter<?>> converters = new ArrayList<>();
        converters.add(this.buildByteConverter());
        converters.add(this.buildJsonConvertor());

        List<HandlerMethodArgumentResolver> resolvers = new ArrayList<>();
        resolvers.add(new PagerArgumentResolver());

        InterceptorBridge bridge = new InterceptorBridge();
        bridge.setInterceptors(interceptors);
        bridge.setResolvers(resolvers);
        bridge.setMessageConverters(converters);

        log.info("build interceptor bridge success!!!");
        return bridge;
    }

    private HttpMessageConverter<?> buildByteConverter() {
        return new ByteArrayHttpMessageConverter();
    }

    private HttpMessageConverter<?> buildJsonConvertor() {
        FastJsonHttpMessageConverter fastJsonConverter = new FastJsonHttpMessageConverter();
        FastJsonConfig config = new FastJsonConfig();
        config.setCharset(StandardCharsets.UTF_8);
        config.setDateFormat("yyyy-MM-dd HH:mm:ss");
        config.setSerializerFeatures(SerializerFeature.WriteMapNullValue, SerializerFeature.WriteNullListAsEmpty);
        fastJsonConverter.setFastJsonConfig(config);
        List<MediaType> list = new ArrayList<>();
        list.add(MediaType.APPLICATION_JSON);
        fastJsonConverter.setSupportedMediaTypes(list);
        return fastJsonConverter;
    }
}
