package com.bocloud.cmp.controller;

import com.bocloud.cmp.entity.Rack;
import com.bocloud.cmp.service.RackService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.ResultTools;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 机柜相关接口
 *
 * @author dmw
 */
@RestController
@RequestMapping("/v1/racks")
@Tag(name = "机柜")
public class RackController {
    @Autowired
    private RackService rackService;

    /**
     * 获取机柜列表
     *
     * @param pager
     * @return 数据结果集
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查看机柜列表")
    @GetMapping
    public GeneralResult<GridBean<Rack>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return rackService.list(pager, context);
    }

    /**
     * 获取机柜列表
     *
     * @param condition
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "通过condition方式查询机柜列表")
    @GetMapping("/condition")
    public GeneralResult<List<Rack>> condition(@RequestParam(value = Common.CONDITION, required = false) String condition,
                                               @Value(Common.REQ_CONTEXT) RequestContext context) {
        return rackService.condition(condition, context);
    }

    /**
     * 添加机柜
     *
     * @param context .getTarget() 操作者ID
     * @return 添加结果
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "添加机柜")
    @PostMapping
    public GeneralResult<Rack> create(@RequestBody Rack rack, @Value(Common.REQ_CONTEXT) RequestContext context) {
        if (null != rack) {
            return rackService.create(rack, context.getTarget());
        } else {
            return ResultTools.formatErrResult();
        }
    }

    /**
     * 修改机柜
     *
     * @param id      需要修改的机柜ID
     * @param context .getTarget() 操作者ID
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "修改机柜")
    @PutMapping(value = "/{id}")
    public GeneralResult<String> modify(@RequestBody Rack rack, @PathVariable(value = Common.ID) Long id,
                                        @Value(Common.REQ_CONTEXT) RequestContext context) {
        if (null != rack) {
            return rackService.modify(rack, context.getTarget());
        } else {
            return ResultTools.formatErrResult();
        }
    }

    /**
     * 移除机柜
     *
     * @param id      机柜ID
     * @param context .getTarget() 操作者ID
     * @return 操作结果
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "移除机柜")
    @DeleteMapping(value = "/{id}")
    public GeneralResult<String> remove(@PathVariable(value = Common.ID) Long id,
                                        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return rackService.remove(id, context.getTarget());
    }

    /**
     * 查看机柜详细信息
     *
     * @param id      机柜ID
     * @param context .getTarget() 操作者ID
     * @return 操作结果
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查看机柜基本信息")
    @GetMapping(value = "/{id}")
    public GeneralResult detail(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context,
                                @RequestParam(value = Common.ACTION, required = false) String action) {
        return rackService.details(id, action);
    }

    /**
     * 查看机柜基本信息
     *
     * @param id 机柜ID
     * @return 操作结果
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查看机柜基本信息")
    @RequestMapping(value = "/{id}/basic", method = {RequestMethod.GET})
    public GeneralResult<Rack> basic(@PathVariable(value = Common.ID) Long id) {
        return rackService.basic(id);
    }

    /**
     * 机柜操作
     *
     * @param context
     * @param params
     * @param action
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "机柜操作")
    @PatchMapping()
    public GeneralResult<String> operations(@Value(Common.REQ_CONTEXT) RequestContext context,
                                            @RequestParam(value = Common.PARAMS, required = false) String params,
                                            @RequestParam(value = Common.ACTION, required = false) String action) {
        return rackService.operations(context, params, action);
    }

}
