package com.bocloud.cmp.controller;

import com.bocloud.cmp.entity.PoolGroup;
import com.bocloud.cmp.entity.PoolGroupRelation;
import com.bocloud.cmp.enums.PoolGroupAction;
import com.bocloud.cmp.service.ResourcePoolService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping("/v1/pool/groups")
@Tag(name = "资源池")
public class ResourcePoolController {

    @Autowired
    private ResourcePoolService resourcePoolService;

    /**
     * 获取资源池列表
     *
     * @param pager
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "创建资源池分组")
    @GetMapping
    public GeneralResult<GridBean<PoolGroup>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return resourcePoolService.list(pager, context);
    }

    /**
     * 获取资源池列表
     *
     * @param condition
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "创建资源池分组")
    @GetMapping("/condition")
    public GeneralResult condition(Pager pager,
                                   @RequestParam(value = Common.CONDITION, required = false) String condition,
                                   @Value(Common.REQ_CONTEXT) RequestContext context) {
        return resourcePoolService.condition(pager, condition, context);
    }


    /**
     * 资源池分组树形展示
     *
     * @param params
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查看资源池列表信息")
    @GetMapping("/tree")
    public GeneralResult list(@RequestParam(value = Common.PARAMS, required = false) String params,
                              @Value(Common.REQ_CONTEXT) RequestContext context) {
        return resourcePoolService.groups(params, context);
    }

    /**
     * 创建资源池
     *
     * @param group
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "创建资源池分组")
    @PostMapping
    public GeneralResult<PoolGroup> create(@RequestBody PoolGroup group,
                                           @Value(Common.REQ_CONTEXT) RequestContext context) {
        group.setCreatorId(context.getTarget());
        group.setMenderId(context.getTarget());
        return resourcePoolService.create(group, context);
    }

    /**
     * 修改资源池
     *
     * @param id
     * @param bean
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "修改资源池分组")
    @PutMapping("/{id}")
    public GeneralResult modify(@PathVariable(value = Common.ID) Long id, @RequestBody PoolGroup bean,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        bean.setId(id);
        return resourcePoolService.modify(bean, context);
    }

    /**
     * 修改资源池状态
     *
     * @param action
     * @return
     */
    @Operation(tags = {"CMC"}, summary = "修改资源池状态")
    @PatchMapping
    public GeneralResult disable(@RequestBody PoolGroupAction action,
                                 @Value(Common.REQ_CONTEXT) RequestContext context) {
        switch (action.getAction()) {
            case able:
                return resourcePoolService.updateStatus(action.getIds(), "NORMAL", context);
            case disable:
                return resourcePoolService.updateStatus(action.getIds(), "ABNORMAL", context);
            default:
                return new GeneralResult(false, "不支持这种操作");
        }

    }

    /**
     * 删除资源池
     *
     * @param id
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "删除资源池分组")
    @DeleteMapping("/{id}")
    public GeneralResult remove(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return resourcePoolService.remove(id, context);
    }

    /**
     * 查询详情
     *
     * @param id
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查看资源池分组详情")
    @GetMapping("/{id}")
    public GeneralResult<PoolGroup> detail(@PathVariable(value = Common.ID) Long id) {
        return resourcePoolService.detail(id);
    }

    /**
     * 资源池容量信息统计
     *
     * @param id
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "资源池容量信息统计")
    @GetMapping("/{id}/capacity")
    public GeneralResult capacity(@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return resourcePoolService.capacity(id, context);
    }

    /**
     * 资源启用、禁用
     *
     * @param id
     * @param poolGroupRelation
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "资源池关联目标机器")
    @PatchMapping("/{id}/resources")
    public GeneralResult resourcesEnabled(@PathVariable(value = Common.ID) Long id, @RequestBody PoolGroupRelation poolGroupRelation) {
        poolGroupRelation.setPoolGroupId(id);
        return resourcePoolService.enabled(poolGroupRelation);
    }

    /**
     * 修改资源池状态
     *
     * @param action
     * @return
     */
    @Operation(tags = {"CMC"}, summary = "修改资源池状态")
    @PatchMapping("region-relation")
    public GeneralResult modifyRegionRelation(@RequestBody PoolGroup group,
                                              @Value(Common.REQ_CONTEXT) RequestContext context) {
        return resourcePoolService.modifyRegionRelation(group.getRelationIds(), group.getIsUsed(), context);
    }
}
