package com.bocloud.cmp.repository;

import com.bocloud.cmp.entity.OSTenant;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * openstack底层租户Repository实现
 *
 * @author wangyu
 * @version 4.2.1-SNAPSHOT
 * @since 2019/11/14
 */
@Repository("osTenantRepository")
public class OSTenantRepository extends BasicGenericDao<OSTenant, Long> {

	public OSTenantRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
		super(jdbcTemplate, npJdbcTemplate, service);
	}

	/**
	 * 查询openstack租户列表
	 *
	 * @param pager 分页参数
	 * @return 成功或失败
	 * @throws Exception
	 */

	public List<OSTenant> list(Pager pager) {
		String sql = "select a.*, t.name tenant_name from os_tenant a left join tenant t on a.tenant_id = t.id where a.is_deleted = 0";
		sql = getQueryBuilder().buildRaw(sql, pager, "a");
		Map<String, Object> param = getQueryBuilder().getParam(pager.getParams());
		return this.list(OSTenant.class, sql, param);
	}

	public List<OSTenant> list(int page, int rows, List<Param> params, Map<String, String> sorter) {
		String sql = "select a.*, t.name tenant_name from os_tenant a left join tenant t on a.tenant_id = t.id where a.is_deleted = 0";
		sql = getQueryBuilder().buildRaw(sql, new Pager(page, rows, params, sorter), "a");
		Map<String, Object> paramMap = getQueryBuilder().getParam(params);
		return this.list(OSTenant.class, sql, paramMap);
	}

	/**
	 * 查询openstack租户列表
	 *
	 * @param params
	 * @return 成功或失败
	 * @throws Exception
	 */

	public List<OSTenant> list(List<Param> params) {
		String sql = "select a.*,b.name creator from os_tenant a left join users b on b.id = a.creator_id where a.is_deleted = 0 ";
		sql = getQueryBuilder().buildRaw(sql, new Pager(1, Integer.MAX_VALUE, new ArrayList<>(), new HashMap<>(8)), "a");
		return this.list(OSTenant.class, sql, new HashMap<>(8));
	}

	/**
	 * 获取openstack租户数量
	 *
	 * @param params
	 * @return 成功或失败
	 * @throws Exception
	 */

	public int count(List<Param> params) {
		String sql = "select count(id) from os_tenant where is_deleted = 0 ";
		sql = getQueryBuilder().build(sql, params, null, "");
		Map<String, Object> param = getQueryBuilder().getParam(params);
		return this.countQuery(sql, param).intValue();
	}

	/**
	 * 删除openstack租户
	 *
	 * @param id
	 * @return 成功或失败
	 * @throws Exception
	 */

	public boolean remove(Long id) {
		String sql = "delete from os_tenant where id = :id";
		Map<String, Object> params = MapTools.simpleMap("id", id);
		return this.execute(sql, params) > 0;
	}

	/**
	 * 查询openstack租户详情
	 *
	 * @param id
	 * @return 成功或失败
	 * @throws Exception
	 */

	public OSTenant query(Long id) {
		String sql = "select a.*, t.name tenant_name from os_tenant a left join tenant t on a.tenant_id = t.id where a.id = :id and a.is_deleted = 0";
		List<OSTenant> osTenants = this.list(OSTenant.class, sql, MapTools.simpleMap("id", id));
		if (ListTool.isEmpty(osTenants)) {
			return null;
		}
		return osTenants.get(0);
	}

	/**
	 * 根据云平台id查询
	 *
	 * @param vendorId 平台id
	 * @return 成功或失败
	 * @throws Exception
	 */

	public List<OSTenant> listByVid(Long vendorId) {
		String sql = "select a.*,b.name creator from os_tenant a left join users b on b.id = a.creator_id where a.vendor_id = :vendorId and a.is_deleted = 0";
		return this.list(OSTenant.class, sql, MapTools.simpleMap(Common.VENDORID, vendorId));
	}

	/**
	 * 根据云平台id删除
	 *
	 * @param vendorId 平台id
	 * @return 成功或失败
	 * @throws Exception
	 */

	public boolean removeByVid(Long vendorId) {
		String sql = "delete from os_tenant where vendor_id = :vendorId";
		Map<String, Object> params = MapTools.simpleMap(Common.VENDORID, vendorId);
		return this.execute(sql, params) > 0;
	}

	/**
	 * 根据名称查询
	 *
	 * @param name
	 * @param vendorId 平台id
	 * @return 成功或失败
	 * @throws Exception
	 */
	public OSTenant getByName(String name, Long vendorId) {
		String sql = "select a.*,b.name creator from os_tenant a left join users b on b.id = a.creator_id where a.name = :name and a.vendor_id = :vendorId and a.is_deleted = 0";
		Map<String, Object> paramMap = MapTools.simpleMap(Common.NAME, name);
		paramMap.put(Common.VENDORID, vendorId);
		List<OSTenant> osTenants = this.list(OSTenant.class, sql, paramMap);
		if (ListTool.isEmpty(osTenants)) {
			return null;
		}
		return osTenants.get(0);
	}

	public OSTenant getByUuid(String tenantUuid) {
		String sql = "select a.*,b.name creator from os_tenant a left join users b on b.id = a.creator_id "
				+ "where a.tenant_uuid = :tenantUuid and a.is_deleted = 0";
		Map<String, Object> paramMap = MapTools.simpleMap("tenantUuid", tenantUuid);
		List<OSTenant> osTenants = this.list(OSTenant.class, sql, paramMap);
		if (ListTool.isEmpty(osTenants)) {
			return null;
		}
		return osTenants.get(0);
	}

	public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) {
		String sql = "select a.id,a.name,a.vendor_id,a.authentication from os_tenant a where a.is_deleted = 0";
		sql = getQueryBuilder().build(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
		Map<String, Object> paramMap = getQueryBuilder().getParam(params);
		List<OSTenant> osTenants = this.list(OSTenant.class, sql, paramMap);
		List<SimpleBean> beans = new ArrayList<>();
		for (OSTenant osTenant : osTenants) {
			beans.add(new SimpleBean(osTenant.getId(), osTenant.getName()));
		}
		return beans;
	}

	public OSTenant getByAk(Long vendorId, String ak, String sk) {
		String sql = "select * from os_tenant where authentication like :ak and is_deleted = 0 and authentication like :sk and vendor_id = :vendorId";
		Map<String, Object> params = MapTools.simpleMap("ak", "%" + ak + "%");
		params.put("sk", "%" + sk + "%");
		params.put("vendorId", vendorId);
		List<OSTenant> list = this.list(OSTenant.class, sql, params);
		if (list.isEmpty()) {
			return null;
		}
		return list.get(0);
	}

	public OSTenant getByTenantUuid(String tenantUuid, Long vendorId) {
		String sql = "select a.*,b.name creator from os_tenant a left join users b on b.id = a.creator_id where a.tenant_uuid = :tenantUuid and a.vendor_id = :vendorId";
		Map<String, Object> paramMap = MapTools.simpleMap("tenantUuid", tenantUuid);
		paramMap.put(Common.VENDORID, vendorId);
		List<OSTenant> osTenants = this.list(OSTenant.class, sql, paramMap);
		if (ListTool.isEmpty(osTenants)) {
			return null;
		}
		return osTenants.get(0);
	}

	public OSTenant getByTenantId(Long tenantId, Long vendorId) {
		if (null == tenantId || 0 == tenantId) {
			return null;
		}
		String sql = "select a.* from os_tenant a where a.tenant_id = :tenantId and a.vendor_id = :vendorId";
		Map<String, Object> paramMap = MapTools.simpleMap("tenantId", tenantId);
		paramMap.put(Common.VENDORID, vendorId);
		List<OSTenant> osTenants = this.list(OSTenant.class, sql, paramMap);
		if (ListTool.isEmpty(osTenants)) {
			return null;
		}
		return osTenants.get(0);
	}

}
