package com.bocloud.cmp.scheduler;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Component;

import com.bocloud.cmp.entity.CloudVendor;
import com.bocloud.cmp.entity.User;
import com.bocloud.cmp.enums.BocloudStatus;
import com.bocloud.cmp.repository.CloudVendorRepository;
import com.bocloud.cmp.service.CloudVendorService;
import com.bocloud.cmp.util.ResourceEventPublisher;
import com.gitee.starblues.integration.user.PluginUser;
import com.megatron.common.model.OperateResult;
import com.megatron.common.model.OperateResult.OperateCategory;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.RequestContext.Catalog;
import com.megatron.common.model.Result;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.core.CurrentService;

import lombok.extern.slf4j.Slf4j;

/**
 * 云供应商连接状态监听定时任务
 *
 * @author wangyu
 * @version 1.0
 * @since 2017年10月17日
 */
@Component
@Slf4j
public class ProviderMonitorSchedule {

	@Autowired
	private CurrentService service;
	@Autowired
	private CloudVendorRepository cloudVendorRepository;
	@Autowired
	private ResourceEventPublisher resourceEventPublisher;
	@Autowired
	private ThreadPoolTaskExecutor taskExecutor;
	@Autowired
	private PluginUser pluginUser;

	/**
	 * 轮询所有云平台连接状态，如果发现连接异常则发送消息给管理员
	 */
	@Scheduled(cron = "${provider.monitor.scheduler:0 0/5 * * * ?}")
	protected void monitor() {
		if (!service.isLeader()) {
			return;
		}
		log.info("当前服务leader状态：{}", service.isLeader());
		taskExecutor.execute(() -> {
			List<CloudVendor> vendors = cloudVendorRepository.listAll();
			for (CloudVendor vendor : vendors) {
				Result result;
				try {
					CloudVendorService pluginCloudVendorService = (CloudVendorService) pluginUser
							.getBean(vendor.getType().toLowerCase(), "cloudVendorService");
					if (null == pluginCloudVendorService) {
						log.info("{}[{}] === 平台认证结果：{} == {}", vendor.getType(), vendor.getName(), false, "暂未找到该类型插件");
						continue;
					}
					result = pluginCloudVendorService.monitor(vendor);
				} catch (Exception e) {
					result = new Result(false, e.getMessage());
				}
				if (null == result) {
					return;
				}
				log.info("{}[{}] === {} == {}", vendor.getType(), vendor.getName(), result.isSuccess(),
						result.isSuccess() ? "认证成功" : "认证失败：" + result.getMessage());
				String operate = vendor.getType().toLowerCase() + ".provider";
				String content = "云平台【" + vendor.getName() + "】连接异常!";
				if (result.isSuccess() && BocloudStatus.Basic.ABNORMAL.name().equals(vendor.getStatus())) {// 连接正常
					vendor.setStatus(BocloudStatus.Basic.NORMAL.name());// 云平台连接正常
					content = "云平台【" + vendor.getName() + "】连接正常!";
					this.updateVendor(vendor, null, operate, content, result);
				} else if (result.isFailed() && BocloudStatus.Basic.NORMAL.name().equals(vendor.getStatus())) {// 平台连接正常
					vendor.setStatus(BocloudStatus.Basic.ABNORMAL.name());// 云平台连接异常
					this.updateVendor(vendor, null, operate, content, result);
				}
			}
		});
	}

	/**
	 * 更新云平台
	 *
	 * @param vendor
	 * @param users
	 * @param operate
	 * @param content
	 * @param result
	 */
	private void updateVendor(CloudVendor vendor, List<User> users, String operate, String content, Result result) {
		cloudVendorRepository.update(vendor);
		if (null != users && users.size() > 0) {
			for (User user : users) {
				RequestContext context = new RequestContext(Catalog.Manager, user.getId(), 0L, 0L, null);
				resourceEventPublisher.send(new OperateResult(result.isSuccess(), content, operate,
						MapTools.simpleMap(Common.VENDORID, vendor.getId()), OperateCategory.ResourceEvent, context));
			}
		}
		// 给admin发消息
		RequestContext context = new RequestContext(Catalog.Manager, 1L, 0L, 0L, null);
		resourceEventPublisher.send(new OperateResult(result.isSuccess(), content, operate,
				MapTools.simpleMap(Common.VENDORID, vendor.getId()), OperateCategory.ResourceEvent, context));
	}

}
