package com.bocloud.cmp.scheduler;

import java.util.List;

import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.entity.CloudVendor;
import com.bocloud.cmp.repository.CloudVendorRepository;
import com.bocloud.cmp.service.CloudVendorService;
import com.gitee.starblues.integration.user.PluginUser;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.ListTool;
import com.megatron.framework.core.CurrentService;

import lombok.extern.slf4j.Slf4j;

/**
 * 云平台统计定时任务
 *
 * @author wangyu
 * @version 4.2.1-SNAPSHOT
 * @since 2019/11/4
 */
@Component
@Slf4j
public class VendorStatisticsScheduler {

	@Autowired
	private CloudVendorRepository cloudVendorRepository;
	@Autowired
	private CurrentService service;
	@Autowired
	private PluginUser pluginUser;

	/**
	 * 每10分钟定时跑一次任务
	 *
	 * @throws Exception
	 */
	@Scheduled(cron = "${vendor.statistics.scheduler:0 1/10 * * * ?}")
	protected void process() throws Exception {
		if (!service.isLeader()) {
			return;
		}
		List<CloudVendor> vendors = cloudVendorRepository.vendorStatusList();
		if (ListTool.isEmpty(vendors)) {
			return;
		}
		for (CloudVendor cloudVendor : vendors) {
			try {
				CloudVendorService pluginCloudVendorService = (CloudVendorService) pluginUser
						.getBean(cloudVendor.getType().toLowerCase(), "cloudVendorService");
				if (null == pluginCloudVendorService) {
					log.info("平台【" + cloudVendor.getName() + "】统计失败，暂未查询到【" + cloudVendor.getType() + "】类型插件");
					continue;
				}
				GeneralResult summaryResult = pluginCloudVendorService.summary(cloudVendor);
				if (summaryResult.isFailed()) {
					log.info("平台【" + cloudVendor.getName() + "】数据统计失败：" + summaryResult.getMessage());
				}
			} catch (Exception e) {
				log.error("云平台[{}]执行summary事件失败！cause:{}", cloudVendor.getName(), e.toString());
			}
		}
	}

	/**
	 * 监听任务用来重新统计云平台使用情况
	 *
	 * @param message
	 * @throws Exception
	 */
	@RabbitListener(admin = "rabbitAdmin", queues = "vendor.statistics.queue", autoStartup = "true")
	public void onMessage(Message message) throws Exception {
		if (!service.isLeader()) {
			return;
		}
		try {
			String content = new String(message.getBody(), "UTF-8");
			JSONObject result = JSONObject.parseObject(content);
			if (null == result) {
				return;
			}
			Long vendorId = result.getLong(Common.VENDORID);
			if (null == vendorId || 0L == vendorId) {
				return;
			}
			CloudVendor vendor = cloudVendorRepository.query(vendorId);
			if (null == vendor) {
				return;
			}
			// 重新统计云平台使用情况
			CloudVendorService pluginCloudVendorService = (CloudVendorService) pluginUser
					.getBean(vendor.getType().toLowerCase(), "cloudVendorService");
			GeneralResult summaryResult = pluginCloudVendorService.summary(vendor);
			if (null != summaryResult) {
				log.info(summaryResult.getMessage());
			}
		} catch (Exception e) {
			log.info("平台定时任务统计异常：" + e);
		}
	}

}
