package com.bocloud.cmp.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.entity.CloudVendor;
import com.bocloud.cmp.entity.PoolGroup;
import com.bocloud.cmp.entity.PoolGroupZone;
import com.bocloud.cmp.entity.Project;
import com.bocloud.cmp.entity.Region;
import com.bocloud.cmp.entity.TenantPool;
import com.bocloud.cmp.entity.Zone;
import com.bocloud.cmp.internal.SmsDictionaryService;
import com.bocloud.cmp.internal.SmsProjectService;
import com.bocloud.cmp.internal.SmsTenantService;
import com.bocloud.cmp.repository.CloudServerRepository;
import com.bocloud.cmp.repository.CloudVendorRepository;
import com.bocloud.cmp.repository.FlavorRepository;
import com.bocloud.cmp.repository.ImageRepository;
import com.bocloud.cmp.repository.NetworkCardRepository;
import com.bocloud.cmp.repository.PoolGroupRepository;
import com.bocloud.cmp.repository.PoolGroupZoneRepository;
import com.bocloud.cmp.repository.RegionRepository;
import com.bocloud.cmp.repository.ResourceRepository;
import com.bocloud.cmp.repository.ServerRepository;
import com.bocloud.cmp.repository.ZoneRepository;
import com.bocloud.cmp.service.RegionService;
import com.google.common.collect.Maps;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.JSONTools;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.ResultTools;

import lombok.extern.slf4j.Slf4j;

/**
 * 区域Service接口实现(子区域为zone)
 *
 * @author dongkai
 */
@Service("regionService")
@Slf4j
public class RegionServiceImpl implements RegionService {

	@Autowired
	private RegionRepository regionRepository;
	@Autowired
	private ZoneRepository zoneRepository;
	@Autowired
	private CloudVendorRepository cloudVendorRepository;
	@Autowired
	private PoolGroupRepository poolGroupRepository;
	@Autowired
	private SmsProjectService smsProjectService;
	@Autowired
	private SmsTenantService smsTenantService;
	@Autowired
	private PoolGroupZoneRepository poolGroupZoneRepository;
	@Autowired
    protected SmsDictionaryService smsDictionaryService;
	@Autowired
    protected ServerRepository serverRepository;
    @Autowired
    protected CloudServerRepository cloudServerRepository;
    @Autowired
    protected ImageRepository imageRepository;
    @Autowired
    protected NetworkCardRepository networkCardRepository;
    @Autowired
    protected ResourceRepository resourceRepository;
    @Autowired
    protected FlavorRepository flavorRepository;

	@Override
	public GeneralResult<List<Region>> list(Long vendorId) {
		try {
			List<Region> list = regionRepository.list(vendorId);
			return new GeneralResult<>(true, list, "查询成功!");
		} catch (Exception e) {
			log.error("list region failure:", e);
			return new GeneralResult<>(false, "查询失败!");
		}
	}

	@Override
	public GeneralResult<List<Zone>> listZone(Long vendorId, String regionId) {
		try {
			CloudVendor vendor = cloudVendorRepository.query(vendorId);
			Assert.notNull(vendor, "云平台不存在");
			List<Zone> list = zoneRepository.listByRegionId(regionId, vendorId);
			return new GeneralResult<>(true, list, "查询成功!");
		} catch (Exception e) {
			log.error("list region failure:", e);
			return new GeneralResult<>(false, "查询失败!");
		}
	}

	@Override
	public GeneralResult<List<PoolGroupZone>> listRegionByGroup(JSONObject params) {
		try {
			if (null == params) {
				return new GeneralResult<>(false, "参数格式错误");
			}
			Long poolGroupId = params.getLong("poolGroupId");
			PoolGroup poolGroup = poolGroupRepository.query(poolGroupId);
			if (null == poolGroup) {
				log.error("资源池不存在!");
				return new GeneralResult<>(false, "资源池不存在");
			}
			List<PoolGroupZone> poolGroupZones = poolGroupZoneRepository.listByGroupId(poolGroup.getVendorId(),
					poolGroup.getId(), null, null);
			String resourceType = params.getString("resourceType");
			if(!StringUtils.isEmpty(resourceType)) {
				List<Region> regionList = regionRepository.list(poolGroup.getVendorId(), resourceType);
				List<String> regionNameList = regionList.stream().map(Region::getName)
						.collect(Collectors.toList());
				if(!ListTool.isEmpty(regionNameList)) {
					poolGroupZones.removeIf(item -> !regionNameList.contains(item.getRegionId()));
				}
			}
			return new GeneralResult<>(true, poolGroupZones, "查询成功!");
		} catch (Exception e) {
			log.error("list region failure:", e);
			return new GeneralResult<>(false, "查询失败!");
		}
	}

	@Override
	public GeneralResult<List<Region>> listByBusiness(JSONObject params, RequestContext context) {
		try {
			Long vendorId = params.getLong(Common.VENDORID);
			GeneralResult<Project> projResult = smsProjectService.detail(context.getProject());
			if (projResult.isFailed() || null == projResult.getData()) {
				return new GeneralResult<>(false, "当前用户所在项目不存在");
			}
			List<TenantPool> pools = smsTenantService.listPoolByTenant(context.getTenant());
			if (ListTool.isEmpty(pools)) {
				return new GeneralResult<>(false, "未找到任何云平台");
			}
			Map<Long, Region> regionMaps = Maps.newHashMap();
			for (TenantPool pool : pools) {
				List<Region> regions = regionRepository.listByGroup(pool.getPoolGroupId(), vendorId);
				if (null != regions && regions.size() > 0) {
					for (Region region : regions) {
						regionMaps.put(region.getId(), region);
					}
				}
			}
			return new GeneralResult<>(true, new ArrayList<>(regionMaps.values()), "查询成功");
		} catch (Exception e) {
			log.error("Get error message：", e);
			return new GeneralResult<>(false, "查询失败");
		}
	}

	@Override
	public GeneralResult condition(String params, RequestContext context) {
		JSONObject object = JSONTools.isJSONObj(params);
		if (null == object) {
			return ResultTools.formatErrResult();
		}
		String condition = object.getString("condition");
		if ("listregionbygroup".equalsIgnoreCase(condition)) {
			return this.listRegionByGroup(object);
		} else if ("listRegionByBusiness".equalsIgnoreCase(condition)) {
			return this.listByBusiness(object, context);
		} else if ("listByTenant".equalsIgnoreCase(condition)) {
			return this.listByTenant(object, context);
		} else {
			return ResultTools.formatErrResult();
		}
	}

	/**
	 * 根据租户的资源池查询公有云的区域 可用区
	 * 
	 * @param object
	 * @param context
	 * @return
	 */
	public GeneralResult<List<Region>> listByTenant(JSONObject object, RequestContext context) {
		try {
			List<TenantPool> pools = smsTenantService.listPoolByTenant(context.getTenant());
			if (ListTool.isEmpty(pools)) {
				return new GeneralResult<>(false, "当前租户未授权资源池！");
			}
			String vendorType = object.getString("vendorType");
			if (StringUtils.isEmpty(vendorType)) {
				return new GeneralResult<>(false, "平台类型参数异常！");
			}
			List<Long> ids = pools.stream().map(TenantPool::getPoolGroupId).collect(Collectors.toList());
			List<PoolGroup> poolGroups = poolGroupRepository.listByIds(ids, vendorType);

			Map<String, Region> regionMap = new HashMap<>();
			for (PoolGroup poolGroup : poolGroups) {
				String region = poolGroup.getRegion();
				String availablitiyZone = poolGroup.getAvailablitiyZone();

				Region reg = null;
				if (regionMap.containsKey(region)) {
					reg = regionMap.get(region);
				} else {
					reg = regionRepository.getByRegionId(region, poolGroup.getVendorId());
				}
				if (null == reg) {
					continue;
				}
				Set<String> zoneIds = reg.getZoneIds();
				if (null == zoneIds) {
					zoneIds = new HashSet<>();
				}
				zoneIds.add(availablitiyZone);
				reg.setZoneIds(zoneIds);
				regionMap.put(region, reg);
			}
			if (regionMap.values().isEmpty()) {
				return new GeneralResult<>(false, "当前租户暂无该服务可用区域！");
			}
			List<Region> regionList = new ArrayList<>();
			// 获取对象 并 去重
			Set<Entry<String, Region>> entrySet = regionMap.entrySet();
			for (Entry<String, Region> entry : entrySet) {
				Region region = entry.getValue();
				Set<String> zoneIds = region.getZoneIds();
				List<Zone> zones = zoneRepository.listByZoneIds(region.getRegionId(), new ArrayList<>(zoneIds),
						region.getVendorId());
				if (!ListTool.isEmpty(zones)) {
					region.setZones(zones);
				}
				regionList.add(region);
			}
			return new GeneralResult<>(true, regionList, "查询成功");
		} catch (Exception e) {
			log.error("Get error message：", e);
			return new GeneralResult<>(false, "查询失败");
		}
	}

	@Override
	public GeneralResult zones(Long id, Long vendorId, String type) {
		return null;
	}
	
	@Override
	public GeneralResult dicToMap(String vendorType) {
		return null;
	}

	/**
	 * 删除区域下的资源
	 * @param regionId
	 * @param vendorId
	 * @param userId
	 * @throws Exception
	 */
	@Override
	public void removeResources(String regionId, Long vendorId, Long userId) throws Exception {
		return ;
	}
	
	/**
	 * 同步云平台下的region
	 * @param vendor
	 * @param userId
	 * @param currentRegions
	 * @throws Exception
	 */
	@Override
	public GeneralResult syncRegion(CloudVendor vendor, Long userId, List<String> currentRegions) throws Exception {
		return null;
	}

	@Override
	public GeneralResult<List<Region>> list(Long vendorId, String resourceType) {
		try {
			List<Region> list = regionRepository.list(vendorId, resourceType);
			return new GeneralResult<>(true, list, "查询成功!");
		} catch (Exception e) {
			log.error("list region failure:", e);
			return new GeneralResult<>(false, "查询失败!");
		}
	}
	
}
