package com.bocloud.cmp.service.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.bocloud.cmp.entity.VendorSyncRecord;
import com.bocloud.cmp.repository.VendorSyncRecordRepository;
import com.bocloud.cmp.service.VendorSyncRecordService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.Result;
import com.megatron.common.utils.Common;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import com.google.common.collect.Maps;

import lombok.extern.slf4j.Slf4j;

/**
 * @author admin
 */
@Service
@Slf4j
public class VendorSyncRecordServiceImpl implements VendorSyncRecordService {

	@Autowired
    private VendorSyncRecordRepository vendorSyncRecordRepository;
	@Autowired
    private LockFactory lockFactory;

    @Override
    public GeneralResult<GridBean<VendorSyncRecord>> list(Pager pager) {
        GridBean<VendorSyncRecord> gridBean;
        try {
            Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap());
            sorter.put("endTime", Common.ONE);
            List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
            int total = this.vendorSyncRecordRepository.count(params);
            List<VendorSyncRecord> list = this.vendorSyncRecordRepository.list(pager.getPage(), pager.getRows(), params, sorter);
            for (VendorSyncRecord record : list) {
                double cost = (double) record.getCost();
                record.setCostStr(dateConvert(cost));
            }
            gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new GeneralResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("Query vendor sync record list error:", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }
    
    public static String dateConvert(double millisecond) {
        long time = (long) millisecond / 1000;
        long hour = (time % (3600 * 24)) / 3600;
        long min = (time % (3600 * 24)) % 3600 / 60;
        long second = (time % (3600 * 24)) % 3600 % 60;
        if (hour == 0) {
            if (min == 0) {
                return second + "秒";
            } else {
                return min + "分" + second + "秒";
            }
        } else {
            return hour + "时" + min + "分" + second + "秒";
        }
    }


    @Override
    public Result save(VendorSyncRecord record) {
        try {
            vendorSyncRecordRepository.save(record);
        } catch (Exception e) {
            log.error("Save vendor sync record error:", e);
        }
        return new Result(true, "保存成功");
    }

    @Override
    public Result remove(Long id) {
        String path = VendorSyncRecord.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            VendorSyncRecord object = this.vendorSyncRecordRepository.query(id);
            if (null == object) {
                log.warn("Vendor sync record does not exist!");
                return new Result(false, "数据不存在");
            }
            this.vendorSyncRecordRepository.delete(id);
            return new Result(true, "删除成功");
        } catch (Exception e) {
            log.error("Remove vendor sync record error:", e);
            return new Result(false, "删除失败");
        }
    }

    @Override
    public GeneralResult<VendorSyncRecord> detail(Long id) {
        try {
            VendorSyncRecord record = this.vendorSyncRecordRepository.query(id);
            if (null == record) {
                return new GeneralResult<>(false, "数据不存在");
            }
            return new GeneralResult<>(true, record, "查询成功");
        } catch (Exception e) {
            log.error("Get vendor sync record error：", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    public Result removeByVendor(Long vendorId) {
        String path = VendorSyncRecord.class.getSimpleName() + "_" + vendorId;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            VendorSyncRecord object = this.vendorSyncRecordRepository.queryByVendor(vendorId);
            if (null == object) {
                log.warn("Vendor sync record does not exist!");
                return new Result(false, "数据不存在");
            }
            this.vendorSyncRecordRepository.deleteByVendor(vendorId);
            return new Result(true, "删除成功");
        } catch (Exception e) {
            log.error("Remove vendor sync record error:", e);
            return new Result(false, "删除失败");
        }
    }

    @Override
    public Result removeByTime(String startTime, String endTime, Long vendorId) {
        try {
            vendorSyncRecordRepository.deleteByTime(startTime, endTime, vendorId);
        } catch (Exception e) {
            log.error("Remove vendor sync record error:", e);
            return new Result(false, "删除失败");
        }
        return new Result(true, "删除成功");
    }

}
